#include <iostream>
#include <string>
#include <stdexcept>
#include <map>

#include "include/tcon_dev.h"

int main(int argc, char *argv[]) {
    std::map<short, std::string> error_to_string = {
            { LIBTCON_SUCCESS, "Success" },
            { LIBTCON_ERROR_IO, "Input / Output error" },
            { LIBTCON_ERROR_INVALID_PARAM, "Invalid parameters" },
            { LIBTCON_ERROR_ACCESS, "Access denied (insufficient permission)" },
            { LIBTCON_ERROR_NO_DEVICE, "No such device" },
            { LIBTCON_ERROR_NOT_FOUND, "Entity not found" },
            { LIBTCON_ERROR_BUSY, "Resource busy" },
            { LIBTCON_ERROR_TIMEOUT, "Operation timed out" },
            { LIBTCON_ERROR_OVERFLOW, "Overflow" },
            { LIBTCON_ERROR_PIPE, "Pipe error" },
            { LIBTCON_ERROR_INTERRUPTED, "System call interrupted" },
            { LIBTCON_ERROR_NO_MEM, "Insufficient memory" },
            { LIBTCON_ERROR_NOT_SUPPORTED, "Operation not supported or unimplemented on this platform" },
            { LIBTCON_ERROR_OTHER, "Other error" },
        };

    std::cout << "ejump_vcom - Sets up VCOM for connected eJump T-CON board" << std::endl;
    std::cout << std::endl;

    if(argc != 2) {
        std::cout << "Usage (assuming VCOM of -1600 mV):" << std::endl;
        std::cout << "sudo ./ejump_vcom 1600" << std::endl;
        return LIBTCON_SUCCESS;
    }

    short vcom_value;
    try {
        vcom_value = std::stoi(argv[1]);
        vcom_value = abs(vcom_value);

        if(vcom_value > 5110) {
            throw std::out_of_range{"VCOM value unreasonably large!"};
        }
    } catch(std::invalid_argument const &ex) {
        std::cerr << "Error: Failed to convert given VCOM value to integer!" << std::endl;
        return 1;
    } catch(std::out_of_range const &ex) {
        std::cerr << "Error: Given VCOM value is out of range!" << std::endl;
        return 1;
    }

    std::cout << "Trying to initialize T-CON board..." << std::endl;
    short board_count = tcon_init();
    if(board_count < 0) {
        std::cerr << "Error: Failed to initialize T-CON board for reason: " << error_to_string[board_count] << std::endl;
        return board_count;
    } else if(board_count == 0) {
        std::cerr << "Error: No T-CON boards detected!" << std::endl;
        return LIBTCON_ERROR_NO_DEVICE;
    } else {
        std::cout << "Successfully initialized T-CON board!" << std::endl;
    }

    // -1: Target T-CON board will be first board to be found via USB
    short board_id = -1;

    if(short retval = tcon_set_vcom(board_id, vcom_value); retval != LIBTCON_SUCCESS) {
        std::cerr << "Error: Failed to set VCOM for reason: " << error_to_string[retval] << std::endl;
        return 3;
    } else {
        std::cout << "Successfully set VCOM to " << (-vcom_value) << "mV!" << std::endl;
    }

    if(short retval = tcon_get_vcom(board_id); retval < 0) {
        std::cerr << "Error: Failed to verify VCOM for reason: " << error_to_string[retval] << std::endl;
        return 4;
    } else if(retval != vcom_value) {
        std::cerr << "Error: Failed to verify VCOM for reason: Values do not match" << std::endl;
        return 4;
    } else {
        std::cout << "Successfully verified VCOM!" << std::endl;
        return 0;
    }
}
