# For killing the process and call parameters
import sys

# With ctypes, one can import a DLL an access its functions
import ctypes

# Used for image processing
from PIL import Image

# Maps any T-CON error code to a descriptive string.
error_to_string = {
    0: "Success",
    -1: "Input / Output error",
    -2: "Invalid parameters",
    -3: "Access denied (insufficient permissions)",
    -4: "No such device",
    -5: "Entity not found",
    -6: "Resource busy",
    -7: "Operation timed out",
    -8: "Overflow",
    -9: "Pipe error",
    -10: "System call interrupted",
    -11: "Insufficient memory",
    -12: "Operation not supported or unimplemented on this platform",
    -99: "Other error"
}

print("ejump_vcom - Sets up VCOM for connected eJump T-CON board")
print("")

if len(sys.argv) != 2:
    print("Usage (assuming VCOM of -1600 mV):")
    print("sudo ./ejump_vcom 1600")
    sys.exit(0)

try:
    vcom_value = int(sys.argv[1])
    vcom_value = abs(vcom_value)
    if vcom_value > 5110:
        print("Error: Given VCOM value is out of range!")
        sys.exit(1)
except ValueError:
    print("Error: Failed to convert given VCOM value to integer!")
    sys.exit(1)

# Load API library via ctypes
tcon_lib = ctypes.cdll.LoadLibrary("/usr/lib/libtcon-dev_usb.so")

# Before we can communicate with the T-CON board, the interface
# has to be initialized. The return value will be the number of
# T-CON boards detected. If return value is 0 or less, an error
# occured (in case of negative values, the error code's meaning
# will be in accordance with the dictionary error_to_string).
print("Trying to initialize T-CON board...")
tcon_num = tcon_lib.tcon_init()
if tcon_num < 0:
    print("Error: Failed to initialize T-CON board for reason:", error_to_string.get(tcon_num, "Unknown Error"))
    sys.exit(tcon_num)
elif tcon_num == 0:
    print("Error: No T-CON boards detected!")
    sys.exit(-4)
else:
    print("Successfully initialized T-CON board!")

# -1: Target T-CON board will be first board to be found via USB
board_id = -1

retval = tcon_lib.tcon_set_vcom(board_id, vcom_value)
if retval < 0:
    print("Error: Failed to set VCOM for reason:", error_to_string.get(tcon_num, "Unknown Error"))
    sys.exit(retval)
else:
    print("Successfully set VCOM to ", -vcom_value, "mV!", sep = '')

retval = tcon_lib.tcon_get_vcom(board_id)
if retval < 0:
    print("Error: Failed to verify VCOM for reason:", error_to_string.get(tcon_num, "Unknown Error"))
    sys.exit(retval)
elif retval != vcom_value:
    print("Error: Failed to verify VCOM for reason: Values do not match")
    sys.exit(-99)
else:
    print("Successfully verified VCOM!")
    sys.exit(0)
